﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace WoodFoxWeiQi.UI
{
	public partial class WeiQiView : UserControl
	{
		WeiQiController _controller = null;
		WeiQiModel _model = null;

		public WeiQiView(WeiQiController controller, WeiQiModel model)
		{
			_controller = controller;
			_model = model;

			// 订阅 model 的更新事件以获得 UI 更新的通知
			_model.BoardUpdated += new EventHandler<BoardUpdateEventArgs>(_model_BoardUpdated);
		}
		
		public void CreateView()
		{
			InitializeComponent();

			canvasBoard.MouseLeftButtonDown += canvasBoard_MouseLeftButtonDown;
			canvasBoard.SizeChanged += canvasBoard_SizeChanged;

			btnGo.Click += btnGo_Click;
			chkShowAxisLabels.Checked += chkShowAxisLabels_Checked;
			chkShowAxisLabels.Unchecked += chkShowAxisLabels_Checked;

			chkShowNumberLabels.Checked += new RoutedEventHandler(chkShowNumberLabels_Checked);
			chkShowNumberLabels.Unchecked += new RoutedEventHandler(chkShowNumberLabels_Checked);
		}
			

		#region Fields
		private readonly Brush brush_White = new SolidColorBrush(Colors.White);
		private readonly Brush brush_Black = new SolidColorBrush(Colors.Black);

		private readonly List<TextBlock> yAxisLabels = new List<TextBlock>(20);
		private readonly List<TextBlock> xAxisLabels = new List<TextBlock>(20);

		double stepLabelFontSize = 10.0; // 用于显示棋步的文字大小

		double boardSize;	// 棋盘宽度（不包含坐标）
		double cellSize;	// 网格宽度
		double starSize;	// 星位的小圆点的直径
		double stoneSize;	// 棋子直径

		// 用于保存在某个棋盘位置的棋子元素
		Dictionary<Position, UIElement> _stones = new Dictionary<Position, UIElement>();

		// 保存数字标签的列表
		Dictionary<Position, UIElement> numbers = new Dictionary<Position, UIElement>();

		#endregion

		// 在某个地方放置一个某颜色的棋子
		void SetStone(Position pos, Stone stone)
		{
			// 移掉老的棋子（如果有）
			if (_stones.ContainsKey(pos))
			{
				UIElement element = _stones[pos];
				_stones.Remove(pos);
				canvasBoard.Children.Remove(element);
			}
			// 添加新棋子
			UIElement newStone = null;
			if (stone == Stone.Black)
			{
				newStone = BuildBlackStone(pos);
			}
			else if (stone == Stone.White)
			{
				newStone = BuildWhiteStone(pos);
			}
			if (newStone != null)
			{
				canvasBoard.Children.Add(newStone);
				_stones.Add(pos, newStone);
			}
		}

		// 创建黑棋
		UIElement BuildBlackStone(Position pos)
		{
			return BuildCircle(stoneSize, pos.X * cellSize, pos.Y * cellSize, 
				brush_Black, brush_Black);			
		}

		// 创建白棋
		UIElement BuildWhiteStone(Position pos)
		{
			return BuildCircle(stoneSize, pos.X * cellSize, pos.Y * cellSize, 
				brush_Black, brush_White);
		}

		// 创建棋步数字标签
		UIElement BuildStepNumberLabel(int step, Position pos)
		{
			if (numbers.ContainsKey(pos))
			{
				var old = numbers[pos];
				numbers.Remove(pos);
				canvasBoard.Children.Remove(old);
			}

			var stone = _model.GetStone(pos);
			if (stone == Stone.None)
				return null;

			// 用适当的反色显示数字标号
			Brush brush = (stone == Stone.Black) ? brush_White : brush_Black;

			var result = BuildLabel(step.ToString(), brush, stepLabelFontSize, 
				pos.X * cellSize, pos.Y * cellSize);

			result.Visibility = NumbersVisible ? Visibility.Visible : Visibility.Collapsed;

			numbers.Add(pos, result);

			return result;
		}
		
		// 显示或隐藏棋步
		void ShowSteps(bool show)
		{
			foreach (var lbl in numbers.Values)
			{
				lbl.Visibility = show ? Visibility.Visible : Visibility.Collapsed;
			}
		}

		void _model_BoardUpdated(object sender, BoardUpdateEventArgs e)
		{
			// 更新 View
			foreach (Position pos in e.ChangedPositions)
			{
				var stone = _model.GetStone(pos);
				SetStone(pos, stone);
				
				int step = _model.GetLastStepNumberByPosition(pos);
				BuildStepNumberLabel(step, pos);
			}
		}

		// 因为 Canvas 的尺寸是根据父控件的尺寸在运行时计算得到的，所以需要在 SizeChanged 方法里
		// 才能获得其实际尺寸
		void canvasBoard_SizeChanged(object sender, SizeChangedEventArgs e)
		{
			canvasBoard.Children.Clear();
			CreateBoardElements();
		}
        
		// 棋盘上的鼠标点击事件
		void canvasBoard_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
		{
			var p = e.GetPosition(canvasBoard);
			
			// 转换绝对坐标为相对位置信息
			Position pos;

			pos.X = (int)Math.Round(p.X / cellSize);
			pos.Y = (int)Math.Round(p.Y / cellSize);

			// 调用 Controller
			_controller.ClickOnBoard(pos);
		}

		void btnGo_Click(object sender, RoutedEventArgs e)
		{
			// 游戏开始
			_controller.DealCommand("Start");
		}

		// 显示或隐藏坐标轴
		void chkShowAxisLabels_Checked(object sender, RoutedEventArgs e)
		{
			var show = chkShowAxisLabels.IsChecked.HasValue && chkShowAxisLabels.IsChecked.Value;
			foreach (var label in xAxisLabels.Union(yAxisLabels))
			{
				label.Visibility = show ? Visibility.Visible : Visibility.Collapsed;
			}
		}

		void chkShowNumberLabels_Checked(object sender, RoutedEventArgs e)
		{
			ShowSteps(NumbersVisible);
		}

		bool NumbersVisible
		{
			get
			{
				return chkShowNumberLabels.IsChecked.HasValue && chkShowNumberLabels.IsChecked.Value;
			}
		}

		#region Builder methods for children elements

		// 创建棋盘上的网格线，星位，坐标等显示元素
		void CreateBoardElements()
		{
			CalculateSizes();

			BuildGridLines();

			BuildStarPointMarks();

			BuildXAxisLabels();

			BuildYAxisLabels();
		}

		// 计算必要的一些尺寸定义值
		void CalculateSizes()
		{
			// 确保使用一个正方形区域作为棋盘显示区域
			boardSize = Math.Min(canvasBoard.ActualHeight, canvasBoard.ActualWidth);

			// 根据棋盘尺寸计算出相应的其他尺寸
			cellSize = boardSize / 20;
			starSize = cellSize / 4;
			stoneSize = cellSize * 0.8;
		}

		// 添加网格线
		void BuildGridLines()
		{
			for (var i = 1; i <= 19; i++)
			{
				// 添加水平网格线
				BuildLine(cellSize, cellSize * i, cellSize * 19, cellSize * i);

				// 添加垂直网格线
				BuildLine(cellSize * i, cellSize, cellSize * i, cellSize * 19);
			}
		}

		// 添加9个星位的标志
		void BuildStarPointMarks()
		{
			for (var i = 4; i <= 16; i += 6)
			{
				for (var j = 4; j <= 16; j += 6)
				{
					var star = BuildCircle(starSize, i * cellSize, j * cellSize, brush_Black, brush_Black);
					canvasBoard.Children.Add(star);
				}
			}
		}

		// 画横坐标
		void BuildXAxisLabels()
		{
			for (var i = 1; i <= 19; i++)
			{
				var lbl = BuildLabel(i.ToString(), brush_Black, 11.0, i * cellSize, cellSize / 2);

				xAxisLabels.Add(lbl);
			}
		}

		// 画纵坐标
		void BuildYAxisLabels()
		{
			var c = 'A';
			for (var i = 1; i <= 19; i++)
			{
				var text = (c++).ToString();
				var lbl = BuildLabel(text, brush_Black, 11.0, cellSize / 2, i * cellSize);

				yAxisLabels.Add(lbl);
			}
		}
		#endregion

		#region Basic builder methods

		Line BuildLine(double x1, double y1, double x2, double y2)
		{
			var line = new Line {
				X1 = x1,
				X2 = x2,
				Y1 = y1,
				Y2 = y2,
				Stroke = brush_Black,
				StrokeThickness = 1.0
			};
			canvasBoard.Children.Add(line);

			return line;
		}

		Ellipse BuildCircle(double diameter, double centerX, double centerY, Brush stroke, Brush fill)
		{
			var ellipse = new Ellipse { Stroke = stroke, Fill = fill };
			ellipse.Width = ellipse.Height = diameter;
			ellipse.SetValue(Canvas.LeftProperty, centerX - diameter / 2);
			ellipse.SetValue(Canvas.TopProperty, centerY - diameter / 2);

			return ellipse;
		}

		// 创建 Label
		TextBlock BuildLabel(string text, Brush foreground, double fontSize,
			double centerX, double centerY)
		{
			var lbl = new TextBlock {
				FontSize = fontSize,
				FontWeight = FontWeights.Thin,
				Text = text,
				Foreground = foreground
			};

			lbl.SetValue(Canvas.LeftProperty, centerX - lbl.ActualWidth / 2);
			lbl.SetValue(Canvas.TopProperty, centerY - lbl.ActualHeight / 2);

			canvasBoard.Children.Add(lbl);

			return lbl;
		}
		#endregion
	}
}
